SystemOrganization addCategory: #'Ice-Tests'!

TestResource subclass: #ICTestResource
	instanceVariableNames: 'fakeClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICTestResource methodsFor: 'running' stamp: 'DeboraFortini 9/30/2011 11:19'!
setUp
	PackageOrganizer default 
		packageNamed: 'Unclassified' 
		ifAbsent: [ PackageOrganizer default registerPackageNamed: 'Unclassified' ].  
	PackageOrganizer default 
		packageNamed: 'FakeExtension' 
		ifAbsent: [ PackageOrganizer default registerPackageNamed: 'FakeExtension' ].  
	fakeClass := Class subclass: #FakeClass.  
	fakeClass 
		compileSilently: 'fakeMethod ^42' 
		classified: #'*fakeExtension'.! !

!ICTestResource methodsFor: 'running' stamp: 'DeboraFortini 10/2/2011 17:18'!
tearDown   
	fakeClass removeFromSystem: false.
	PackageOrganizer default unregisterPackageNamed: 'FakeExtension'.
	PackageOrganizer default unregisterPackageNamed: 'Unclassified'.
	 ! !

TestCase subclass: #ICBaseTest
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICBaseTest classSide methodsFor: 'as yet unclassified' stamp: 'DeboraFortini 9/7/2011 12:20'!
resources 
	^Array with: ICTestResource! !

!ICBaseTest methodsFor: 'tests' stamp: 'DeboraFortini 10/2/2011 17:35'!
assertClassExists: aSymbol
	self assert: (Smalltalk at: aSymbol ifAbsent: [ nil ]) notNil	! !

!ICBaseTest methodsFor: 'tests' stamp: 'DeboraFortini 11/13/2011 19:47'!
ensureRemoveClass: aSymbol
	Smalltalk at: aSymbol ifPresent: [ :class | class removeFromSystem: false]. 
	! !

!ICBaseTest methodsFor: 'accessing' stamp: 'EstebanLorenzano 1/17/2011 13:19'!
fakeClass 
	^Smalltalk 
		at: #FakeClass 
		ifAbsent: [ nil ]! !

!ICBaseTest methodsFor: 'accessing' stamp: 'EstebanLorenzano 1/17/2011 13:21'!
repository 
	^repository! !

!ICBaseTest methodsFor: 'running' stamp: 'DeboraFortini 9/29/2011 19:30'!
setUp  
	repository := ICRepository on: (FSFilesystem memory).
	! !

!ICBaseTest methodsFor: 'accessing' stamp: 'DeboraFortini 11/19/2011 12:12'!
string: aString existsOnFile: aReference 
	|contents |
	self assert:  aReference exists.
	contents := aReference readStreamDo: [ :stream | stream contents asString ].
	^contents includesSubString: aString.! !

!ICBaseTest methodsFor: 'tests' stamp: 'DeboraFortini 10/2/2011 17:40'!
withTempClass: aSymbol do: aBlock
	| tempClass |

	tempClass := Class subclass: aSymbol.  
	[ aBlock value: tempClass ] ensure:  [ self ensureRemoveClass: aSymbol]
	! !

ICBaseTest subclass: #ICChangeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICChangeTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 10:50'!
testRemoveClassChange
	| newChange |
	
	(self repository storage / 'Unclassified' / 'FakeClass.st')
		in: [ :me | me parent ensureDirectory ];
		fileStreamDo: [ :stream | stream nextPutAll: 'test' ].  
	newChange := ICRemoveClassChange className: #FakeClass category: 'Unclassified'.
	newChange applyOn: self repository.
	
	self deny: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. ! !

!ICChangeTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 10:50'!
testUpdateClassChange
	| newChange |
	newChange := ICUpdateClassChange className: #FakeClass category: 'Unclassified'.
	newChange applyOn: self repository.
	
	self assert: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. ! !

!ICChangeTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testUpdateExtensionsChange
	| newChange |
	
	self repository addPackageNamed: 'FakeExtension'.  
	newChange := ICUpdateExtensionsChange 
		className: #FakeClass 
		category: 'Extensions'
		packages: (self repository containers select: [ :each | each isExtension not ]).
	newChange applyOn: self repository.
	
	self assert: (self repository storage / 'Extensions' / 'FakeClass.st') exists. ! !

ICBaseTest subclass: #ICContainerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICContainerTest methodsFor: 'tests' stamp: 'DeboraFortini 10/18/2011 16:09'!
testActionAddClass
	| container |
	
	container := ICPackageContainer for: (PackageOrganizer default 
		packageNamed: 'Unclassified' 
		ifAbsent: [ self error ]).
	container addChangeRecord: (ICAddClassChangeRecord for: self fakeClass).
	
	self assert: container changeRecords size = 1.
	self assert: container allChangeRecords size = 1.
	self assert: container effectiveChanges size = 1.
	
	container addChangeRecord: (ICUpdateClassChangeRecord for: self fakeClass).
	
	self assert: container changeRecords size = 1.
	self assert: container effectiveChanges size = 1.! !

!ICContainerTest methodsFor: 'tests' stamp: 'DeboraFortini 10/18/2011 16:10'!
testActionAddExtension
	| container |
	
	container := ICExtensionContainer for: (PackageOrganizer default 
		packageNamed: 'FakeExtension' 
		ifAbsent: [ self error ]).
	container addChangeRecord: (ICAddMethodChangeRecord for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod)).
	self assert: container changeRecords size = 1.
	self assert: container allChangeRecords size = 1.! !

ICBaseTest subclass: #ICEventsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICEventsTest methodsFor: 'tests' stamp: 'DeboraFortini 10/3/2011 00:34'!
setUp 
	super setUp.
	ICRepositoryManager uniqueInstance addRepository: self repository.
	
	self repository addPackageNamed: 'Unclassified'.
	self repository push.
! !

!ICEventsTest methodsFor: 'tests' stamp: 'DeboraFortini 9/29/2011 19:32'!
tearDown 
	ICRepositoryManager uniqueInstance removeRepository: self repository! !

!ICEventsTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testAddClassEvent
	
	self withTempClass:  #FakeNewAddedClass do: [ :fakeNewAddedClass |
		self assert: self repository isDirty.
		self assert: self repository containersWithChangeRecords size = 1.
		self assert: (self repository changeRecords  
			anySatisfy: [:each | each class = ICAddClassChangeRecord and: [each name =#FakeNewAddedClass]]). 
	].! !

!ICEventsTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:19'!
testAddMethodEvent
	
	self withTempClass:  #FakeNewAddedClass do: [ :fakeNewAddedClass |
		self repository push. 
	  	fakeNewAddedClass compile: 'fakeAddedMethod ^42'. 
		
	  	self assert: (self repository changeRecords  
			anySatisfy: [:each | each class = ICAddMethodChangeRecord and: [each name =#fakeAddedMethod]])
	]. 
	! !

!ICEventsTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testRemoveClassEvent
	| classToRemove |
	
	self withTempClass: #FakeClassToRemove do: [ :fakeClass |
		"add a new class"
		self assert: self repository isDirty.
		self repository push.
		
		"delete the class added and pushed before"
		self repository pull.
		(Smalltalk at: #FakeClassToRemove) removeFromSystem: true.
			
		self assert: self repository containersWithChangeRecords size = 1.
		self assert: (self repository changeRecords  
			anySatisfy: [:each | each class = ICRemoveClassChangeRecord and: [each name =#FakeClassToRemove]]). 
	].! !

!ICEventsTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testRemoveMethodEvent
	| classForDeleteMethod |
	
	self withTempClass:  #FakeClassToRomeveMethod do: [ :fakeClass |
		"add method"
		 fakeClass compile: 'fakeAddedMethod ^42'.
		self repository push.
		
		"remove method"
		fakeClass removeSelector: #fakeAddedMethod.
		
		self assert: self repository isDirty.				
		self assert: self repository containersWithChangeRecords size = 1.
		self assert: (self repository changeRecords  
			anySatisfy: [:each | each class = ICRemoveMethodChangeRecord and: [each name =#fakeAddedMethod]])
		
	].! !

!ICEventsTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testUpdateClassEvent
	
	self withTempClass:  #FakeNewAddedClass do: [ :fakeAddedClass |
		self assert: self repository isDirty.
		self repository push.
			
		"update class"
		fakeAddedClass addInstVarNamed: 'aVariable'.
			
		self assert: self repository containersWithChangeRecords size = 1.
		self assert: (self repository changeRecords  
			anySatisfy: [:each | each class = ICUpdateClassChangeRecord and: [each name =#FakeNewAddedClass]]) 
		
	].! !

!ICEventsTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testUpdateMethodEvent
	
	self withTempClass:  #FakeNewAddedClass do: [ :fakeAddedClass |
		self assert: self repository isDirty.
		self repository push.
			
		"add method"
		 fakeAddedClass compileSilently: 'fakeAddedMethod ^42'.
		self repository push.
		self deny: self repository isDirty.
		
		"modify method"
		fakeAddedClass compile: 'fakeAddedMethod ^43'.	
		self assert: self repository isDirty.	
			
		self assert: self repository containersWithChangeRecords size = 1.
		self assert: (self repository changeRecords  
			anySatisfy: [:each | each class = ICUpdateMethodChangeRecord and: [each name =#fakeAddedMethod]])
		
	].! !

ICBaseTest subclass: #ICOperationTest
	instanceVariableNames: 'fakeClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICOperationTest methodsFor: 'tests' stamp: 'DeboraFortini 10/2/2011 17:38'!
testPullClass
	self withTempClass: #FakeClass2 do: [ : fakeClass2 |
		self repository addPackageNamed: 'Unclassified'.
		self repository push.
	
		fakeClass2 removeFromSystem: false.
		self repository removeHistoryAt: #FakeClass2.
	  
		self repository pull.
	
		self assertClassExists: #FakeClass2.
	] 
	! !

!ICOperationTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 11:42'!
testPullExtension
	
	self withTempClass: #FakeClassWithExtensionMethod do: [ :aFakeClass |
		aFakeClass 
			compileSilently: 'fakeAddedMethod ^42' 
			classified: #'*fakeExtension'. 
		self repository addPackageNamed: 'FakeExtension'.
	
		self repository addChangeRecord: (ICAddMethodChangeRecord for: (MethodReference 
				class: aFakeClass 
				selector: #fakeAddedMethod)).
		self repository push.

		self repository removeHistoryAt: aFakeClass name.
		aFakeClass removeSelector: #fakeAddedMethod.

		self repository pull.
	
		self assert: (aFakeClass selectors includes: #fakeAddedMethod).
	]

	! !

!ICOperationTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 11:42'!
testPullExtensionMethodInExistingClass
	self repository addPackageNamed: 'Unclassified'. 
	
	self withTempClass: #FakeClassWithExtensionMethod do: [ :fakeClassWithExtensionMethod |
		self repository addChangeRecord: (ICAddClassChangeRecord for: fakeClassWithExtensionMethod).
		fakeClassWithExtensionMethod 
			compileSilently: 'fakeExtensionMethodAdded ^42' 
			classified: #'*fakeExtension'.  
		self repository push.
		
		fakeClassWithExtensionMethod removeFromSystem: false.
		self repository removeHistoryAt: #FakeClassWithExtensionMethod.
		  
		self repository pull.
		self assert: ((Smalltalk at: #FakeClassWithExtensionMethod) canUnderstand:  #fakeExtensionMethodAdded).
	]
	! !

!ICOperationTest methodsFor: 'tests' stamp: 'DeboraFortini 10/2/2011 21:08'!
testPullMethodInExistingClass
	| class |

	self repository addPackageNamed: 'Unclassified'.
	
	class := Smalltalk at: #FakeClass. 
	[
		class compileSilently: 'fakeAddedMethod ^42'. 
		self repository push.

		class removeFromSystem: false.
		self repository removeHistoryAt: #FakeClass.
	  
		self repository pull.
		self assert: ((Smalltalk at: #FakeClass) canUnderstand: #fakeAddedMethod).
	] 
	ensure: [ (Smalltalk at: #FakeClass) removeSelector: #fakeAddedMethod ].	! !

!ICOperationTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testPushAddClass
	self repository addPackageNamed: 'Unclassified'.
	self repository push.
	
	self assert: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. ! !

!ICOperationTest methodsFor: 'tests' stamp: 'DeboraFortini 11/19/2011 12:13'!
testPushAddExtensionMethodInExistingClass
	| reference | 
	
	self withTempClass: #FakeClassWithExtensionMethod do: [ :aFakeClass |
		self repository addPackageNamed: 'FakeExtension'.

		aFakeClass 
			compileSilently: 'fakeExtensionMethodAdded ^42' 
			classified: #'*fakeExtension'. 
	
		self repository addChangeRecord: (ICAddMethodChangeRecord for: (MethodReference 
				class: aFakeClass 
				selector: #fakeExtensionMethodAdded)).
		self repository push.

		self assert: (self string: 'fakeExtensionMethodAdded' existsOnFile: self repository storage / 'Extensions' / 'FakeClassWithExtensionMethod.st').
	]

	! !

!ICOperationTest methodsFor: 'tests' stamp: 'DeboraFortini 11/19/2011 12:17'!
testPushAddMethodInExistingClass
	| container methodReference |

	self repository addPackageNamed: 'Unclassified'.

	self withTempClass:  #FakeClass2 do: [ :addedClass | 
		self repository addChangeRecord: (ICAddClassChangeRecord for: addedClass).
		self repository push.
		
		self deny: (self string: 'fakeAddedMethod' existsOnFile: self repository storage / 'Unclassified' / 'FakeClass2.st'). 

		"add a new method and push"
		addedClass compileSilently: 'fakeAddedMethod ^42'. 
		methodReference := MethodReference class: addedClass selector: #fakeAddedMethod.
		self repository addChangeRecord: (ICAddMethodChangeRecord for: methodReference).	
		self repository push.

		self assert: (self string: 'fakeAddedMethod' existsOnFile: self repository storage / 'Unclassified' / 'FakeClass2.st'). 
	]
	! !

!ICOperationTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testPushExtensions
	self repository addPackageNamed: 'FakeExtension'.
	self repository push.
	
	self assert: (self repository storage / 'Extensions' / 'FakeClass.st') exists. 
	self deny: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. ! !

!ICOperationTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 11:42'!
testPushManyActions
	"Tests many actions. 
	  WARNING I'm using Kernel package to check the correct push, but this can change any time"
	self repository containers add: (ICPackageContainer for: (PackageOrganizer default 
		packageNamed: 'Kernel'
		ifAbsent: [ self error ])).
	self repository containers add: (ICPackageContainer for: (PackageOrganizer default 
		packageNamed: 'Unclassified'
		ifAbsent: [ self error ])).
	.
	self repository addChangeRecord: (ICUpdateClassChangeRecord for: self fakeClass).  
	self repository addChangeRecord: (ICUpdateClassChangeRecord for: Object). 
	self repository addChangeRecord: (ICUpdateClassChangeRecord for: UndefinedObject). 
	self repository addChangeRecord: (ICUpdateMethodChangeRecord for: (MethodReference 
		class: SequenceableCollection 
		selector: #putOn:)).
	self repository push.
	self assert: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. 
	self assert: (self repository storage / 'Kernel-Objects' / 'Object.st') exists. 
	self assert: (self repository storage / 'Kernel-Objects' / 'UndefinedObject.st') exists. 
	self assert: (self repository storage / 'Extensions' / 'SequenceableCollection.st') exists. ! !

!ICOperationTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 11:42'!
testPushUpdateClass
	self repository containers add: (ICPackageContainer for: (PackageOrganizer default 
		packageNamed: 'Unclassified'
		ifAbsent: [ self error ])).
	self repository addChangeRecord: (ICUpdateClassChangeRecord for: self fakeClass).  
	self repository push.
	
	self assert: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. ! !

ICBaseTest subclass: #ICPullTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICPullTest methodsFor: 'as yet unclassified' stamp: 'DeboraFortini 11/21/2011 10:21'!
testConflict
	[
	| c1 reference rep2|
	
	c1  := Class subclass: #C1.
	self repository addPackageNamed: 'Unclassified'.
	c1 compileSilently: 'm42^42'.
	self repository push.
     
	self assert: (self string: 'm42' existsOnFile: self repository storage /'Unclassified'/'C1.st').
	
	c1 compileSilently: 'm43^43'.
	
	"Add a repository with a diferent version of the code"
	rep2 := ICRepository on: (FSFilesystem memory).	
	rep2 addPackageNamed: 'Unclassified'.
	rep2 push.
	
	self assert: (self string: 'm43' existsOnFile: rep2 storage /'Unclassified'/'C1.st').
	
	c1 removeSelectorSilently: #m43.
	
	self repository  pull.
	self assert: (self repository status = ICRepository statusConflict).
	self assert: (self string: 'm43' existsOnFile: self repository  storage /'Unclassified'/'C1.st').
	
	] ensure: [ self ensureRemoveClass:  #C1.]! !

!ICPullTest methodsFor: 'as yet unclassified' stamp: 'DeboraFortini 11/17/2011 17:02'!
testPullClassesWithCycleReferences
	[| c1 c2 |
	
	"create a herachy, where subclass references superclass"
	c1 := Class subclass: #C1.
	c2 := c1 subclass: #C2.
	c1 compileSilently: 'm1 ^C2'.
	
	self repository addPackageNamed: 'Unclassified'.
	self repository push.
	
	c1 removeFromSystem: false.
	c2 removeFromSystem: false.
	self repository removeHistoryAt: #C1.
	self repository removeHistoryAt: #C2.
	
	self repository pull.
	c1 := Smalltalk at: #C1.
	c2 := Smalltalk at: #C2.
	
	self assert: c2 superclass = c1.
	self assert: c1 new m1 = c2.
	] 
	ensure: [
		self ensureRemoveClass: #C1. 
		self ensureRemoveClass: #C2 ].
	! !

ICBaseTest subclass: #ICRepositoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testAddActionAddClass
	| newAction |
	
	self repository addPackageNamed: 'Unclassified'.
	
	newAction := ICAddClassChangeRecord for: self fakeClass.
	self repository addChangeRecord: newAction.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithChangeRecords size = 1.
	self assert: (self repository changeRecords includes: newAction).
	! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testAddActionAddMethod
	| newAction |
	
	self repository addPackageNamed: 'Unclassified'.
	
	newAction := ICAddMethodChangeRecord for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod).
	self repository addChangeRecord: newAction.
	self assert: self repository isDirty.
	self assert: self repository containersWithChangeRecords size = 1.
	self assert: (self repository changeRecords includes: newAction).
	! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testAddActionRemoveClass
	| newAction |
	
	self repository addPackageNamed: 'Unclassified'.   
	
	newAction := ICRemoveClassChangeRecord for: self fakeClass.
	self repository addChangeRecord: newAction.	
	
	self assert: self repository isDirty.
	self assert: self repository containersWithChangeRecords size = 1.
	self assert: (self repository changeRecords includes: newAction).! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testAddActionRemoveMethod
	| newAction |
	
	self repository addPackageNamed: 'Unclassified'.   
	
	newAction := ICRemoveMethodChangeRecord for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod).
	self repository addChangeRecord: newAction.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithChangeRecords size = 1.
	self assert: (self repository changeRecords includes: newAction).! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testAddActionUpdateAfterAddClass
	| newAction1 newAction2 |
	
	self repository addPackageNamed: 'Unclassified'.   
	
	newAction1 := ICAddClassChangeRecord for: self fakeClass.
	newAction2 := ICUpdateClassChangeRecord for: self fakeClass.
	self repository addChangeRecord: newAction1.
	self repository addChangeRecord: newAction2.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithChangeRecords size = 1.
	self assert: (self repository changeRecords includes:  newAction1).
	self assert: (self repository changeRecords includes:  newAction2).! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testAddActionUpdateAfterAddMethod
	| newAction1 newAction2 |

	self repository addPackageNamed: 'Unclassified'.   

	newAction1 := ICAddMethodChangeRecord for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod).
	newAction2 := ICUpdateMethodChangeRecord for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod).	
	self repository addChangeRecord: newAction1.	
	self repository addChangeRecord: newAction2.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithChangeRecords size = 1.
	self assert: (self repository changeRecords includes:  newAction1).
	self assert: (self repository changeRecords includes:  newAction2).! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testAddActionUpdateClass
	| newAction |
	
	self repository addPackageNamed: 'Unclassified'.   
	
	newAction := ICUpdateClassChangeRecord for: self fakeClass.
	self repository addChangeRecord: newAction.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithChangeRecords size = 1.
	self assert: (self repository changeRecords includes: newAction).
	! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testAddActionUpdateMethod
	| newAction |
	self repository addPackageNamed: 'Unclassified'.   
	
	newAction := ICUpdateMethodChangeRecord for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod).
	self repository addChangeRecord: newAction.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithChangeRecords size = 1.
	self assert: (self repository changeRecords includes: newAction).
	! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/18/2011 16:11'!
testAddExtensions 
	self repository addPackageNamed: 'FakeExtension'. 
	self assert: self repository containersWithChangeRecords size = 1.
	self assert: self repository extensionContainer allChangeRecords size = 1.
	self assert: self repository extensionContainer allChangeRecords first name = #fakeMethod.
	self assert: self repository extensionContainer allChangeRecords first className = #FakeClass.! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/18/2011 16:12'!
testAddExtensionsThenAddPackage 
	self repository addPackageNamed: 'FakeExtension'. 
	self repository addPackageNamed: 'Unclassified'. 
	
	self assert: self repository extensionContainer allChangeRecords isEmpty. 
! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 14:30'!
testAddPackage 
	| fakeClassToAdd |
	
	fakeClassToAdd := Class subclass: #FakeClassToAdd. 
	[ 
		self repository addPackageNamed: 'Unclassified'. 
		self assert: self repository containersWithChangeRecords size = 1.
		self assert: (self repository changeRecords 
			anySatisfy: [ :each | 
				(each className = 'FakeClassToAdd') and: [ each class = ICAddClassChangeRecord ] ])
	] ensure: [ fakeClassToAdd removeFromSystem: false ]! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/18/2011 16:12'!
testAddPackageThenAddExtensions 
	self repository addPackageNamed: 'Unclassified'. 
	self repository addPackageNamed: 'FakeExtension'. 
	
	self assert: self repository extensionContainer allChangeRecords isEmpty. 
! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/12/2011 20:48'!
testChangesAddAction
	| changes fakeClass2 |
	self repository addPackageNamed:  'Unclassified'.
	self repository addChangeRecord: (ICUpdateClassChangeRecord for: self fakeClass).

	"just one change (one class)"	
	changes := self repository effectiveChanges.
	self assert: changes notEmpty.
	self assert: changes size = 1.
	self assert: changes first class = ICUpdateClassChange.
	self assert: changes first className = #FakeClass.
	self assert: changes first classCategory = 'Unclassified'.
	
	self repository addChangeRecord: (ICUpdateMethodChangeRecord for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod)).  

	"still just one change (one class)"
	changes := self repository effectiveChanges.
	self assert: changes notEmpty.
	self assert: changes size = 1.
	self assert: changes first class = ICUpdateClassChange.
	self assert: changes first className = #FakeClass.
	self assert: changes first classCategory = 'Unclassified'.

	fakeClass2 := Class subclass: #FakeClass2.
	[ 
		self repository addChangeRecord: (ICAddClassChangeRecord for: fakeClass2).

		"Now, two changes (two classes)"
		changes := self repository effectiveChanges.
		self assert: changes notEmpty.
		self assert: changes size = 2.
		self assert: changes last class = ICUpdateClassChange.
		self assert: changes last className = #FakeClass2.
		self assert: changes last classCategory = 'Unclassified'. ]
	ensure: [ fakeClass2 removeFromSystem: false ]  ! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/12/2011 20:48'!
testChangesAddContainer
	| changes |
	
	self repository addPackageNamed: 'FakeExtension'.
	"One change (one class with extensions)"  
	changes := self repository effectiveChanges.
	self assert: changes notEmpty.
	self assert: changes size = 1.
	self assert: changes first class = ICUpdateExtensionsChange.
	self assert: changes first className = #FakeClass.
	self assert: changes first classCategory = 'Extensions'.	
	
	self repository addPackageNamed:  'Unclassified'.
	"One change (one class, no extensions)"
	changes := self repository effectiveChanges.
	self assert: changes notEmpty.
	self assert: changes size = 1.
	self assert: changes first class = ICUpdateClassChange.
	self assert: changes first className = #FakeClass.
	self assert: changes first classCategory = 'Unclassified'.! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 10/4/2011 11:42'!
testStatusOk
	"FSMemoryFilesystem fails to give me valid modification times, I need to use a real repository to 
	 test this"
	| fileRepository reference |

	reference := FSFilesystem disk workingDirectory  / 'repository-test'.
	reference ensureDirectory.
	fileRepository := ICRepository on: reference.
	
	[
		"Just added, FakeClass status is 'Changed'"
		fileRepository addPackageNamed: 'Unclassified'.  
		self assert: fileRepository status = ICRepository statusChanged.
	
		"After push repository is 'Ok'"
		fileRepository push.
		self assert: fileRepository status = ICRepository statusOk.
	
		"After added action, repository is 'Changed'"
		fileRepository addChangeRecord: (ICUpdateClassChangeRecord for: self fakeClass).
		self assert: fileRepository status = ICRepository statusChanged.
		
		"After modify externally smalltalk file, with internal change, repository is 'Conflict'"
		(Delay forSeconds: 1) wait. "Ensure change time (since nanoseconds is not working for now :()"
		(reference / 'test.st') fileStreamDo: [ :stream | stream nextPutAll: 'testing' ].
		self assert: fileRepository status = ICRepository statusConflict ]	
	ensure: [ reference deleteAll ]
	! !

ICBaseTest subclass: #ICSavingChangesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICSavingChangesTest methodsFor: 'as yet unclassified' stamp: 'DeboraFortini 11/21/2011 12:53'!
testSaveChangeTimestamp
	|today|
	today := Date today.
	self withTempClass: #ClassToTestTimestamp do: [ :aClass |
		self repository addPackageNamed: 'Unclassified'.
		self assert: (self repository lastInternalTimeStamp date = today).
		self assert: (self repository lastStorageTimeStamp =  nil).
		
		self repository push.
		self assert: (self repository lastStorageTimeStamp date = today).
	]! !
